# Spice.ai v0.3-alpha

We are excited to announce the release of Spice.ai v0.3-alpha! 🎉

This release adds support for ingestion, automatic encoding, and training of categorical data, enabling more use-cases and datasets beyond just numerical measurements. For example, perhaps you want to learn from data that includes a category of t-shirt sizes, with discrete values, such as small, medium, and large. The v0.3 engine now supports this and automatically encodes the categorical string values into numerical values that the AI engine can use. Also included is a preview of data visualizations in the dashboard, which is helpful for developers as they author Spicepods and dataspaces. 

A special acknowledgment to [@sboorlagadda](https://github.com/sboorlagadda), who submitted the first Spice.ai feature contribution from the community ever! He added the ability to list pods from the CLI with the new `spice pods list` command. Thank you, [@sboorlagadda](https://github.com/sboorlagadda)!!!

If you are new to Spice.ai, check out the [getting started guide](https://docs.spiceai.org/getting-started/) and star [spiceai/spiceai](https://github.com/spiceai/spiceai) on GitHub.

## Highlights in v0.3-alpha

### Categorical data

In v0.1, the runtime and AI engine only supported ingesting numerical data. In v0.2, tagged data was accepted and automatically encoded into fields available for learning. In this release, v0.3, categorical data can now also be ingested and automatically encoded into fields available for learning. This is a *breaking change* with the format of the manifest changing separating numerical measurements and categorical data.

Pre-v0.3, the manifest author specified numerical data using the `fields` node.

In v0.3, numerical data is now specified under `measurements` and categorical data under `categories`. E.g.

```yaml
dataspaces:
  - from: event
    name: stream
    measurements:
      - name: duration
        selector: length_of_time
        fill: none
      - name: guest_count
        selector: num_guests
        fill: none
    categories:
      - name: event_type
        values:
          - dinner
          - party
      - name: target_audience
        values:
          - employees
          - investors
    tags:
      - tagA
      - tagB
```

### Data visualizations preview

A top piece of community feedback was the ability to visualize data. After first running Spice.ai, we'd often hear from developers, "how do I see the data?". A preview of data visualizations is now included in the dashboard on the pod page.

### Listing pods

Once the Spice.ai runtime has started, you can view the loaded pods on the dashboard and fetch them via API call [localhost:8000/api/v0.1/pods](http://localhost:8000/api/v0.1/pods). To make it even easier, we've added the ability to list them via the CLI with the new `spice pods list` command, which shows the list of pods and their manifest paths.

### Coinbase data connector

A new [Coinbase data connector](https://github.com/spiceai/data-components-contrib/blob/trunk/dataconnectors/coinbase/README.md) is included in v0.3, enabling the streaming of live market ticker prices from Coinbase Pro. Enable it by specifying the `coinbase` data connector and providing a list of Coinbase Pro product ids. E.g. "BTC-USD". A new sample which demonstrates is also available with its associated Spicepod available from the spicerack.org registry. Get it with `spice add samples/trader`.

### Tweet Recommendation Quickstart

A new [Tweet Recommendation Quickstart](https://github.com/spiceai/quickstarts/blob/trunk/tweet-recommendation/README.md) has been added. Given past tweet activity and metrics of a given account, this app can recommend when to tweet, comment, or retweet to maximize for like count, interaction rates, and outreach of said given Twitter account.

### Trader Sample

A new [Trader Sample](https://github.com/spiceai/samples/blob/trunk/trader/README.md) has been added in addition to the Trader Quickstart. The sample uses the new [Coinbase data connector](https://github.com/spiceai/data-components-contrib/blob/trunk/dataconnectors/coinbase/README.md) to stream live Coinbase Pro ticker data for learning.

## New in this release

- **Adds** support for ingesting, encoding, and training on categorical data. v0.3 uses one-hot-encoding.
- **Changes** Spicepod manifest fields node to [measurements](docs.spiceai.org/reference/pod/#dataspacesmeasurements) and add the [categories](docs.spiceai.org/reference/pod/#dataspacescategories) node.
- **Adds** the ability to select a field from the source data and map it to a different field name in the dataspace. See [an example for measurements](docs.spiceai.org/reference/pod/#dataspacesmeasurementsselector) in docs.
- **Adds** support for JSON content type when fetching from the `/observations` API. Previously, only CSV was supported.
- **Adds** a preview version of data visualizations to the dashboard. The grid has several limitations, one of which is it currently cannot be resized.
- **Adds** the ability to select which learning algorithm to use via the CLI, the API, and specified in the Spicepod manifest. Possible choices are currently "vpg", Vanilla Policy Gradient and "dql", Deep Q-Learning. Shout out to [@corentin-pro](https://github.com/corentin-pro), who added this feature on his second day on the team!
- **Adds** the ability to list loaded pods with the CLI command spice pods list.
- **Adds** a new [coinbase data connector](https://github.com/spiceai/data-components-contrib/blob/trunk/dataconnectors/coinbase/README.md) for Coinbase Pro market prices.
- **Adds** a new [Tweet Recommendation Quickstart](https://github.com/spiceai/quickstarts/blob/trunk/tweet-recommendation/README.md).
- **Adds** a new [Trader Sample](https://github.com/spiceai/samples/blob/trunk/trader/README.md).
- **Fixes** bug where the `/observations` endpoint was not providing fully qualified field names.
- **Fixes** issue where debugging messages were printed when using spice add.
